;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :compo
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :compo)

(export '(*compo-source* 
          *compo-object*

          circar
          circdr
          format-doc
          *default-format*
          make-and-load
          mydeftype
          type-spec-p
          
          ; Compositions
          
          abstract-compo
          name
          subs
          compo
          ccompo
          context
          context-item
          coercer

          ; Comparators, accessors and copy constructor
          
          is-equal
          compo-lessp
          clone-slots
          clone
          make-constant
          
          ; Realization of compositions
          
          realize
          nrealize
          realize-using-context-item

          ; context items

          atomization
          coercion
          evaluation
          granulation
          integration
          realization
          recursion
          sorting
          selection
          transformation
          unicity

          ; coercion specifiers

          coerce-value
          closestc

          ; Redefinable realization behaviour

          coerce-compo
          evaluate
          granulate-compo
          integrate-compo
          select-compo
          transform
          
          ; Contexts
          
          standard-context
          
          ; Lexical variables
          
          super-compo
          prev-compo
          
          ; sugar

          normalize-compo
          characterizep
          specialized-normalize
          specialized-print
          key->val
          quote-expr
          read-compo
          sugar-compo
          declare-note
          sub-print
          sub-pprint ))

(setf (logical-pathname-translations "compo")
      `(("**;*.*" , (make-pathname
                     :host *host*
                     :device *device*
                     :directory `(,@*compo-root* "compo" :wild-inferiors)
                     :name :wild
                     :type :wild ))) )

(defun make-and-load (object-pathname source-pathname)

"Function
   make-and-load (object-pathname source-pathname)
Arguments
   An object logical path as a string
   A source logical path as a string
Values
   Doesn't return anything. Only used for its side effects.
Purpose
   Loads the file after compiling it when necessary.
Behaviour
   The object pathname is pretty-printed.
   If the *rebuild-all* parameter is not null, or either the file pointed by the object
   pathname does not exist, or the time stamp of the file pointed by the source pathname
   is more recent that the object file ones, the source file is compiled and the result
   file placed at the location and name specified by the object pathname.
   The object file is loaded."
   
  (setf object-pathname (logical-pathname object-pathname))
  (setf source-pathname (logical-pathname source-pathname))
  (pprint (translate-logical-pathname object-pathname))
  (when (or *rebuild-all*

            ; lispworks wants 'file-write-date' to be inserted in a 'with-open-file' form
            
            (< (with-open-file (file (compile-file-pathname
                                      source-pathname
                                      :output-file object-pathname )
                                     :if-does-not-exist nil )
                 (if file (file-write-date file) 0) )
               (with-open-file (file source-pathname) (file-write-date file)) ) )
         (compile-file source-pathname :output-file object-pathname) )
  (load (translate-logical-pathname object-pathname)) )

(defvar *default-format* :text)

(make-and-load "compo:bin;circar"
               "compo:src;circar.lisp" )

(make-and-load "compo:bin;circdr"
               "compo:src;circdr.lisp" )

(make-and-load "compo:bin;closestc"
               "compo:src;coercion-specifiers;closestc.lisp" )

(make-and-load "compo:bin;coerce-value"
               "compo:src;coerce-value.lisp" )

(make-and-load "compo:bin;format-doc"
               "compo:src;format-doc.lisp" )

(make-and-load "compo:bin;mydeftype"
               "compo:src;mydeftype.lisp" )

(make-and-load "compo:bin;abstract-compo"
               "compo:src;compos;abstract-compo.lisp" )

(make-and-load "compo:bin;compo"
               "compo:src;compos;compo.lisp" )

(make-and-load "compo:bin;ccompo"
               "compo:src;compos;ccompo.lisp" )

(make-and-load "compo:bin;context-item"
               "compo:src;context-item.lisp" )

(make-and-load "compo:bin;realize-using-context-item"
               "compo:src;realize-using-context-item.lisp" )

(make-and-load "compo:bin;atomization"
               "compo:src;context-items;atomization.lisp" )

(make-and-load "compo:bin;coercion"
               "compo:src;context-items;coercion.lisp" )

(make-and-load "compo:bin;evaluation"
               "compo:src;context-items;evaluation.lisp" )

(make-and-load "compo:bin;granulation"
               "compo:src;context-items;granulation.lisp" )

(make-and-load "compo:bin;integration"
               "compo:src;context-items;integration.lisp" )

(make-and-load "compo:bin;realization"
               "compo:src;context-items;realization.lisp" )

(make-and-load "compo:bin;recursion"
               "compo:src;context-items;recursion.lisp" )

(make-and-load "compo:bin;selection"
               "compo:src;context-items;selection.lisp" )

(make-and-load "compo:bin;sorting"
               "compo:src;context-items;sorting.lisp" )

(make-and-load "compo:bin;transformation"
               "compo:src;context-items;transformation.lisp" )

(make-and-load "compo:bin;unicity"
               "compo:src;context-items;unicity.lisp" )

(make-and-load "compo:bin;standard-context"
               "compo:src;contexts;standard-context.lisp" )

(make-and-load "compo:bin;is-equal"
               "compo:src;redef;is-equal.lisp" )

(make-and-load "compo:bin;compo-lessp"
               "compo:src;redef;compo-lessp.lisp" )

(make-and-load "compo:bin;make-constant"
               "compo:src;redef;make-constant.lisp" )

(make-and-load "compo:bin;clone-slots"
               "compo:src;redef;clone-slots.lisp" )

(make-and-load "compo:bin;clone"
               "compo:src;clone.lisp" )

(make-and-load "compo:bin;realize"
               "compo:src;realize.lisp" )

(make-and-load "compo:bin;nrealize"
               "compo:src;nrealize.lisp" )

(make-and-load "compo:bin;coerce-compo"
               "compo:src;redef;coerce-compo.lisp" )

(make-and-load "compo:bin;evaluate"
               "compo:src;redef;evaluate.lisp" )

(make-and-load "compo:bin;granulate-compo"
               "compo:src;redef;granulate-compo.lisp" )

(make-and-load "compo:bin;integrate-compo"
               "compo:src;redef;integrate-compo.lisp" )

(make-and-load "compo:bin;select-compo"
               "compo:src;redef;select-compo.lisp" )

(make-and-load "compo:bin;specialized-normalize"
               "compo:src;redef;sugar;specialized-normalize.lisp" )

(make-and-load "compo:bin;specialized-print"
               "compo:src;redef;sugar;specialized-print.lisp" )

(make-and-load "compo:bin;read-compo"
               "compo:src;redef;sugar;read-compo.lisp" )

(make-and-load "compo:bin;sugar-compo"
               "compo:src;redef;sugar;sugar-compo.lisp" )

(make-and-load "compo:bin;transform"
               "compo:src;redef;transform.lisp" )

(make-and-load "compo:bin;key-val"
               "compo:src;sugar;key-val.lisp" )

(make-and-load "compo:bin;normalize-compo"
               "compo:src;sugar;normalize-compo.lisp" )

(make-and-load "compo:bin;declare-note"
               "compo:src;sugar;declare-note.lisp" )

(make-and-load "compo:bin;pprint-compo"
               "compo:src;sugar;pprint-compo.lisp" )

(make-and-load "compo:bin;print-object"
               "compo:src;sugar;print-object.lisp" )

(make-and-load "compo:bin;quote-expr"
               "compo:src;sugar;quote-expr.lisp" )

(make-and-load "compo:bin;sub-pprint"
               "compo:src;sugar;sub-pprint.lisp" )

(make-and-load "compo:bin;sub-print"
               "compo:src;sugar;sub-print.lisp" )

(make-and-load "compo:bin;check-name"
               "compo:src;sugar;error;check-name.lisp" )

(make-and-load "compo:bin;check-reference"
               "compo:src;sugar;error;check-reference.lisp" )

(pushnew :compo *features*)