;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(unless (find-package :note)(make-package :note))
(in-package :note)
(use-package :compo)

(when (or *reload-all*
          (not (find :compo *features*)) )
  (load (make-pathname :host *host*
                       :device *device* 
                       :directory (append *compo-root* '("compo"))
                       :name "package" 
                       :type *source-type*) ) )

(export '(

          *note-source* 
          *note-object*

          ; Notes
          
          abstract-note
          note
          cnote
          hei
          voi
          pos
          dur
          dyn
          coerce-hei
          coerce-voi
          coerce-pos
          coerce-dur
          coerce-dyn
          fermata
          repeat-end
          fermata-repeat-end
          lyrics

          ; Lexical variables
          
          super-hei
          super-dur
          super-pos
          super-voi
          super-dyn
          prev-hei
          prev-pos
          prev-dur
          prev-voi
          prev-dyn
          
          ; pitches

          *default-syntax*
          intern-temperament
          *compo-temperament*
          *pythagorician-temperament*
          *equal-temperament*
          *default-temperament*
          normalize-pitch
          denormalize-pitch
          pitchp
          itp
          itei
          itep
          pti
          ptep
          naturalp
          sharpp
          flatp
          double-sharp-p
          double-flat-p
          funcptf
          ptf-eq-440
          ptf-py-440
          
          ; cleaves
          
          cleavep
          ntoc
          ntoev
          cton
          
          ; durations
          
          durationp
          ntod
          dton

          ; coercion specifiers

          mode
          
          ; Sugar
          
          sugar-note
          compo-help

          *err-prompt*
          specialized-help ))

(proclaim '(special *default-syntax* *default-temperament*))

(setf (logical-pathname-translations "note")
      `(("**;*.*" , (make-pathname :host *host*
                     :device *device*
                     :directory `(,@*compo-root* "note" :wild-inferiors)
                     :name :wild
                     :type :wild ))) )

(make-and-load "note:bin;abstract-note"
               "note:src;notes;abstract-note.lisp" )

(make-and-load "note:bin;note"
               "note:src;notes;note.lisp" )

(make-and-load "note:bin;cnote"
               "note:src;notes;cnote.lisp" )

(make-and-load "note:bin;fermata"
               "note:src;notes;fermata.lisp" )

(make-and-load "note:bin;repeat-end"
               "note:src;notes;repeat-end.lisp" )

(make-and-load "note:bin;fermata-repeat-end"
               "note:src;notes;fermata-repeat-end.lisp" )

(make-and-load "note:bin;lyrics"
               "note:src;notes;lyrics.lisp" )

(make-and-load "note:bin;clone-slots"
               "note:src;redef;clone-slots.lisp" )

(make-and-load "note:bin;coerce-compo"
               "note:src;redef;coerce-compo.lisp" )

(make-and-load "note:bin;compo-lessp"
               "note:src;redef;compo-lessp.lisp" )

(make-and-load "note:bin;make-constant"
               "note:src;redef;make-constant.lisp" )

(make-and-load "note:bin;is-equal"
               "note:src;redef;is-equal.lisp" )

(make-and-load "note:bin;evaluate"
               "note:src;redef;evaluate.lisp" )

(make-and-load "note:bin;granulate-compo"
               "note:src;redef;granulate-compo.lisp" )

(make-and-load "note:bin;integrate-compo"
               "note:src;redef;integrate-compo.lisp" )

(make-and-load "note:bin;select-compo"
               "note:src;redef;select-compo.lisp" )

(make-and-load "note:bin;transform"
               "note:src;redef;transform.lisp" )

(make-and-load "note:bin;cleaves"
               "note:src;cleaves.lisp" )

(make-and-load "note:bin;durations"
               "note:src;durations.lisp" )

(make-and-load "note:bin;mode"
               "note:src;coercion-specifiers;mode.lisp" )

(make-and-load "note:bin;pitch"
               "note:src;pitch.lisp" )

(defparameter *default-syntax* :english

"Parameter
   *default-syntax*
Purpose
   The parameter *default-syntax* is expected to be set to either the keyword :english or
   the keyword :latin. It determines globally for a compo session which syntax is to be
   used for interpreting pitch names (:c :d :e... vs :do :re :mi...) and durations
   (:q (quarter) vs :n (noire)...)." )

(defconstant *compo-temperament*
  (intern-temperament
   '(1200 ; octave
     0 196 392 502 698 894 1090
     86 282 478 588 784 980 1176
     1114 110 306 416 612 808 1004
     172 368 564 674 870 1066 62
     1028 24 220 330 526 722 918 ) )


"Constant
   *compo-temperament*
Purpose
   The compo musical temperament doesn't exist as an historical fact, as it is the case
   for the pythagorician temperament typically. Unlike the equal temperament, it is
   designed in such a way that each pitch among the 35 compo's known pitches has a
   different value, and unlike the pythagorician temperament, altered pitches are closer
   to their homonym natural pitch than to the next adjacent pitch with a different name.
   That is, unlike in the pythagorician temperament, C sharp is closer to C than to
   D in the compo temperament.
   In this temperament, a fifth is conventionally equal to 698, and all the other pitches
   are obtained by adding or subtracting fifth while stepping through the fifth cycle,
   modulo the octaves. For instance, D is just after G in the fifth cycle. So it is equal
   to 698 plus 698 minus one octave (1200) which is 196." )

(defconstant *pythagorician-temperament*
  (intern-temperament
   '(1200 ; octave
     0 204 408 498 702 906 1110
     114 318 522 612 816 1020 24
     1086 90 294 384 588 792 996
     228 432 636 726 930 1134 138
     972 1176 180 270 474 678 882 ) )

"Constant
   *pythagorician-temperament*
Purpose
   The value in cents associated to the pitches are computed as a round of the linearized
   pythagorician intervals, where a fifth is approximatively equal to 702, and all the
   other pitches are obtained by adding or subtracting fifth while stepping through the
   fifth cycle, modulo the octaves. For instance, D is just after G in the fifth cycle.
   So it is equal to 702 plus 702 minus one octave (1200) which is 204." )

(defconstant *equal-temperament*
  (intern-temperament
   '(1200 ; octave
     0 200 400 500 700 900 1100
     100 300 500 600 800 1000 0
     1100 100 300 400 600 800 1000
     200 400 600 700 900 1100 100
     1000 0 200 300 500 700 900 ) )

"Constant
   *equal-temperament*
Purpose
   The value in cents associated to the pitches are computed as a round of the linearized
   intervals in equal temperament, where a semi-tone is equal to the twelfth of an
   octave, that is 100. This is equivallent to midicents except that 0 corresponds to the
   medium C which is 6000 in midicents." )

(defvar *default-temperament* *compo-temperament*

"Variable
   *default-temperament*
Purpose
   The compo temperament is set as the default temperament when starting compo. Changing
   the default temperament is done by setting this variable to another temperament,
   either one of the two ones provided by the system, the pythagorician or the equal
   temperament, or any temperament provided as an extension (see the function
   'intern-temperament')." )

(make-and-load "note:bin;grammar"
               "note:src;grammar.lisp" )

(make-and-load "note:bin;not-a-dur-p"
               "note:src;sugar;error;not-a-dur-p.lisp" )

(make-and-load "note:bin;check-dur"
               "note:src;sugar;error;check-dur.lisp" )

(make-and-load "note:bin;not-a-dur-coercer-p"
               "note:src;sugar;error;not-a-dur-coercer-p.lisp" )

(make-and-load "note:bin;check-dur-coercer"
               "note:src;sugar;error;check-dur-coercer.lisp" )

(make-and-load "note:bin;not-a-hei-p"
               "note:src;sugar;error;not-a-hei-p.lisp" )

(make-and-load "note:bin;check-hei"
               "note:src;sugar;error;check-hei.lisp" )

(make-and-load "note:bin;not-a-hei-coercer-p"
               "note:src;sugar;error;not-a-hei-coercer-p.lisp" )

(make-and-load "note:bin;check-hei-coercer"
               "note:src;sugar;error;check-hei-coercer.lisp" )

(make-and-load "note:bin;check-pos-mod"
               "note:src;sugar;error;check-pos-mod.lisp" )

(make-and-load "note:bin;not-a-value-p"
               "note:src;sugar;error;not-a-value-p.lisp" )

(make-and-load "note:bin;check-pos"
               "note:src;sugar;error;check-pos.lisp" )

(make-and-load "note:bin;not-a-coercer-p"
               "note:src;sugar;error;not-a-coercer-p.lisp" )

(make-and-load "note:bin;check-pos-coercer"
               "note:src;sugar;error;check-pos-coercer.lisp" )

(make-and-load "note:bin;not-a-voi-p"
               "note:src;sugar;error;not-a-voi-p.lisp" )

(make-and-load "note:bin;check-voi"
               "note:src;sugar;error;check-voi.lisp" )

(make-and-load "note:bin;not-a-voi-coercer-p"
               "note:src;sugar;error;not-a-voi-coercer-p.lisp" )

(make-and-load "note:bin;check-voi-coercer"
               "note:src;sugar;error;check-voi-coercer.lisp" )

(make-and-load "note:bin;check-dyn"
               "note:src;sugar;error;check-dyn.lisp" )

(make-and-load "note:bin;check-dyn-coercer"
               "note:src;sugar;error;check-dyn-coercer.lisp" )

(make-and-load "note:bin;check-parameter-key"
               "note:src;sugar;error;check-parameter-key.lisp" )

(make-and-load "note:bin;specialized-help"
               "note:src;sugar;specialized-help.lisp" )

(make-and-load "note:bin;compo-help"
               "note:src;sugar;compo-help.lisp" )

(make-and-load "note:bin;specialized-normalize"
               "note:src;redef;sugar;specialized-normalize.lisp" )

(make-and-load "note:bin;specialized-print"
               "note:src;redef;sugar;specialized-print.lisp" )

(make-and-load "note:bin;sugar-note"
               "note:src;redef;sugar;sugar-note.lisp" )

(pushnew :note *features*)