;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defun durationp (x &key (syntax *default-syntax*))

"Predicate
   durationp (x &key :syntax)
Arguments
   A value to be considered as being or not a duration keyword
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Values
   The argument if it is a duration keyword, nil otherwise.
Behaviour
   If the specified or default syntax is the syntax :english, the keyword is searched
   among the list :q (quarter) :h (half) :w (whole) :dw (double-whole) :8th :16th :32th 
   and :64, each allowed to be followed by one or two points (for example :q. or :8th..).
   If the specified or default syntax is the syntax :latin, the keyword is searched among
   the list :n (noire) :b (blanche) :r (ronde) :dr (double-ronde) :c (croche) :dc
   (double-croche) :tc (triple-croche) and :qc (quadruple-croche), each allowed to be
   followed by one or two points (for example :n. or :c..)."

  (and
   (member x (if (eq syntax :english)
               '(:q :q. :q..
                 :h :h. :h..
                 :w :w. :w..
                 :dw :dw. :dw..
                 :8th :8th. :8th..
                 :16th :16th. :16th..
                 :32th :32th. :32th..
                 :64th :64th. :64th.. )
               
               '(:n :n. :n..
                 :b :b. :b..
                 :r :r. :r..
                 :dr :dr. :dr..
                 :c :c. :c..
                 :dc :dc. :dc..
                 :tc :tc. :tc..
                 :qc :qc. :qc.. ) ) )
   x ) )

(defun ntod (num &key (syntax *default-syntax*))

"Function
   ntod (num &key :syntax)
Arguments
   A number to be translated to a duration keyword
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Values
   The duration keyword expressed in the specified or default syntax, or nil if the
   number is not the ratio between one of the 24 (see 'durationp') known duration
   keywords and the quarter.
Behaviour
   The correspondance is as the following :
   . 1 corresponds to the quarter;
   . each value corresponds to the ratio between a duration and the quarter. For example,
     2 is an half while 1/2 is a 8th and 3/2 a dotted quarter."

  (if (eq syntax :english)
    (cond 
     ((= num 1) :q)
     ((= num 3/2) :q.)
     ((= num 7/4) :q..)
     ((= num 2) :h)
     ((= num 3) :h.)
     ((= num 7/2) :h..)
     ((= num 4) :w)
     ((= num 6) :w.)
     ((= num 7) :w..)
     ((= num 8) :dw)
     ((= num 12) :dw.)
     ((= num 14) :dw..)
     ((= num 1/2) :8th)
     ((= num 3/4) :8th.)
     ((= num 7/8) :8th..)
     ((= num 1/4) :16th)
     ((= num 3/8) :16th.)
     ((= num 7/16) :16th..)
     ((= num 1/8) :32th)
     ((= num 3/16) :32th.)
     ((= num 7/32) :32th..)
     ((= num 1/16) :64th)
     ((= num 3/32) :64th.)
     ((= num 7/64) :64th..)
     (t nil) )

    (cond 
     ((= num 1) :n)
     ((= num 3/2) :n.)
     ((= num 7/4) :n..)
     ((= num 2) :b)
     ((= num 3) :b.)
     ((= num 7/2) :b..)
     ((= num 4) :r)
     ((= num 6) :r.)
     ((= num 7) :r..)
     ((= num 8) :dr)
     ((= num 12) :dr.)
     ((= num 14) :dr..)
     ((= num 1/2) :c)
     ((= num 3/4) :c.)
     ((= num 7/8) :c..)
     ((= num 1/4) :dc)
     ((= num 3/8) :dc.)
     ((= num 7/16) :dc..)
     ((= num 1/8) :tc)
     ((= num 3/16) :tc.)
     ((= num 7/32) :tc..)
     ((= num 1/16) :qc)
     ((= num 3/32) :qc.)
     ((= num 7/64) :qc..)
     (t nil) ) ) )

(defun dton (dur &key (syntax *default-syntax*))

"Function
   dton (dur &key :syntax)
Arguments
   A duration keyword to be translated to a number
   A syntax as a key argument, defaulting to the value of the special variable
   *default-syntax*
Values
   The ratio between the duration keyword and the quarter, or nil if the duration keyword
   is unknown in the specified or default syntax."

  (if (eq syntax :english)
    (cond 
     ((eq dur :q) 1)
     ((eq dur :q.) 3/2)
     ((eq dur :q..) 7/4)
     ((eq dur :h) 2)
     ((eq dur :h.) 3)
     ((eq dur :h..) 7/2)
     ((eq dur :w) 4)
     ((eq dur :w.) 6)
     ((eq dur :w..) 7)
     ((eq dur :dw) 8)
     ((eq dur :dw.) 12)
     ((eq dur :dw..) 14)
     ((eq dur :8th) 1/2)
     ((eq dur :8th.) 3/4)
     ((eq dur :8th..) 7/8)
     ((eq dur :16th) 1/4)
     ((eq dur :16th.) 3/8)
     ((eq dur :16th..) 7/16)
     ((eq dur :32th) 1/8)
     ((eq dur :32th.) 3/16)
     ((eq dur :32th..) 7/32)
     ((eq dur :64th) 1/16)
     ((eq dur :64th.) 3/32)
     ((eq dur :64th..) 7/64)
     (t nil) )

    (cond 
     ((eq dur :n) 1)
     ((eq dur :n.) 3/2)
     ((eq dur :n..) 7/4)
     ((eq dur :b) 2)
     ((eq dur :b.) 3)
     ((eq dur :b..) 7/2)
     ((eq dur :r) 4)
     ((eq dur :r.) 6)
     ((eq dur :r..) 7)
     ((eq dur :dr) 8)
     ((eq dur :dr.) 12)
     ((eq dur :dr..) 14)
     ((eq dur :c) 1/2)
     ((eq dur :c.) 3/4)
     ((eq dur :c..) 7/8)
     ((eq dur :dc) 1/4)
     ((eq dur :dc.) 3/8)
     ((eq dur :dc..) 7/16)
     ((eq dur :tc) 1/8)
     ((eq dur :tc.) 3/16)
     ((eq dur :tc..) 7/32)
     ((eq dur :qc) 1/16)
     ((eq dur :qc.) 3/32)
     ((eq dur :qc..) 7/64)
     (t nil) ) ) )