;;; A Common Lisp implementation of the compo protocol Version 2
;;;
;;; Package :note
;;;
;;; Copyright (C) 2001  Bruno LARTILLOT
;;; 
;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License
;;; as published by the Free Software Foundation; either version 2
;;; of the License, or any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

(in-package :note)

(defmethod evaluate ((note note) super-compo prev-compo)

"Primary method
   evaluate specialized on the note type
Behaviour
   For each numeric property ('hei', 'voi', 'pos', 'dur' and 'dyn'), the supervalue is
   determined as being the corresponding value in the supernote (defaulting to 0 in the
   case of 'hei' being null), or the neutral value for that property (0, 0, 0, 1 and 1
   respectively) if there is no supernote (if the current note is the root of the tree
   being realized).
   For each numeric property, the previous value is determined as being the corresponding
   value in the previous note (defaulting to 0 in the case of 'hei' being null), or 0 if
   there is no previous note (if the current note is either the root of the tree being
   realized or the first note of the subnote list being realized.
   In the value of each numeric property of the current note, each instance of a variable
   (determined by the presence of a super-xxx or a prev-xxx symbol, where xxx is one of
   the numeric property), is bound to the corresponding value determined above. This is
   done by computing a new value where instances of the variables are replaced by their 
   current value (i.e. what is called a shallow binding in computer languages
   implementation).
   Each of the new bound value is 'evaluated' in the sense defined in compo as the
   following (where <<eval>> refers to the standard Lisp evaluation) :
   . If the value is an atom, it is <<eval>>ed;
   . If the value is a non-quoted list which car is an atom, the whole list is intended
   to be a functional call which is <<eval>>ed;
   . If the value is a non-quoted list which car is a cons, its car is appended to its
   cdr. The result is intended to be a functional call which is <<eval>>ed;
   . If the value is a quoted list which car is an atom, the whole list is intended to be
   a coercion specifier, for which each argument (each item of the list except the first)
   are <<eval>>ed;
   . If the value is a quoted list which car is a cons, its car is appended to its
   cdr. the result is intended to be a coercion specifier, for which each argument (each
   item of the list except the first) are <<eval>>ed.
   Each numeric slot in the current note is set to the new evaluated corresponding
   value.
   Returns the current note as a result."

  (let* ((super-hei (if super-compo (or (hei super-compo) 0) 0))
         (super-voi (if super-compo (voi super-compo) 0))
         (super-pos (if super-compo (pos super-compo) 0))
         (super-dur (if super-compo (dur super-compo) 1))
         (super-dyn (if super-compo (dyn super-compo) 1))
         (prev-hei (if prev-compo (or (hei prev-compo) 0) 0))
         (prev-voi (if prev-compo (voi prev-compo) 0))
         (prev-pos (if prev-compo (pos prev-compo) super-pos))
         (prev-dur (if prev-compo (dur prev-compo) 0))
         (prev-dyn (if prev-compo (dyn prev-compo) 0)) )

    (labels ((bind (a)
               (if (atom a)
                 (cond
                  ((eq a 'prev-hei) prev-hei)
                  ((eq a 'prev-voi) prev-voi)
                  ((eq a 'prev-pos) prev-pos)
                  ((eq a 'prev-dur) prev-dur)
                  ((eq a 'prev-dyn) prev-dyn)
                  ((eq a 'super-hei) super-hei)
                  ((eq a 'super-voi) super-voi)
                  ((eq a 'super-pos) super-pos)
                  ((eq a 'super-dur) super-dur)
                  ((eq a 'super-dyn) super-dyn)
                  (t a) )
                 (cons (bind (car a)) (bind (cdr a))) ) )
             (ev (a)
               (if (atom a)
                 (eval a)
                 (if (not (eq (car a) 'quote))
                   (let ((b (if (atom (car a))
                              a
                              (append (car a) (cdr a)) )))
                       (eval (cons (car b) (mapcar #'eval (cdr b)))) )
                   (progn
                     (if (consp (caadr a))
                       (list 'quote
                             (cons (caaadr a)
                                   (mapcar #'eval (append (cdaadr a) (cdadr a))) ) )
                       (list 'quote
                             (cons (caadr a)
                                   (mapcar #'eval (cdadr a)) ) ) ) ) ) ) ) )
      (with-slots (hei voi pos dur dyn) note
        (setf hei (and hei (ev (bind hei))))
        (setf voi (ev (bind voi)))
        (setf pos (ev (bind pos)))
        (setf dur (ev (bind dur)))
        (setf dyn (ev (bind dyn))) ) ) )
  note )